/*
Copyright 2021 ICS-FORTH.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package workflow

import (
	"github.com/fnikolai/frisbee/api/v1alpha1"
	"github.com/pkg/errors"
	"github.com/sirupsen/logrus"
	"sigs.k8s.io/controller-runtime/pkg/client"
)

func (r *Controller) getJob(action v1alpha1.Action) (client.Object, error) {
	logrus.Warn("Handle job ", action.Name)

	switch action.ActionType {
	case "Service":
		return r.service(action)

	case "Cluster":
		return r.cluster(action)

	case "Chaos":
		return r.chaos(action)

	default:
		return nil, errors.Errorf("unknown action %s", action.ActionType)
	}
}

func (r *Controller) service(action v1alpha1.Action) (client.Object, error) {
	var service v1alpha1.Service

	service.SetName(action.Name)

	action.Service.DeepCopyInto(&service.Spec)

	return &service, nil
}

func (r *Controller) cluster(action v1alpha1.Action) (client.Object, error) {
	var cluster v1alpha1.Cluster

	cluster.SetName(action.Name)

	action.Cluster.DeepCopyInto(&cluster.Spec)

	return &cluster, nil
}

func (r *Controller) chaos(action v1alpha1.Action) (client.Object, error) {
	var chaos v1alpha1.Chaos

	if chaos.Spec.Type == v1alpha1.FaultKill {
		if action.DependsOn.Success != nil {
			return nil, errors.Errorf("kill is a inject-only chaos. it does not have success. only running")
		}
	}

	chaos.SetName(action.Name)

	action.Chaos.DeepCopyInto(&chaos.Spec)

	return &chaos, nil
}
